{ nixpkgs ? import <nixpkgs> {}
, compiler ? "default"
, doBenchmark ? false
}:

let

  inherit (nixpkgs) pkgs;

  sdl = with haskellPackages; callPackage(
    { mkDerivation, base, bytestring, deepseq, exceptions, linear, SDL2
    , StateVar, stdenv, text, transformers, vector, weigh
    }:
    mkDerivation {
      pname = "sdl2";
      version = "2.5.2.0";
      sha256 = "9c4110b1e772739bf62c641d375665f9d0af3ad64ed73b24ef407a82e7648fa1";
      isLibrary = true;
      isExecutable = true;
      enableSeparateDataOutput = true;
      libraryHaskellDepends = [
        base bytestring exceptions linear StateVar text transformers vector
      ];
      librarySystemDepends = [ SDL2 ];
      libraryPkgconfigDepends = [ SDL2 ];
      testHaskellDepends = [ base deepseq linear vector weigh ];
      doHaddock = false;
      doCheck = false;
      description = "Both high- and low-level bindings to the SDL library (version 2.0.6+).";
      license = stdenv.lib.licenses.bsd3;
    }) {};

  nanovg = with haskellPackages; callPackage (
    { mkDerivation, base, bytestring, c2hs, containers, glew
    , hspec, inline-c, libGL, libGLU, QuickCheck, stdenv, text, vector
    }:
    mkDerivation {
      pname = "nanovg";
      version = "0.6.0.0";
      src = ../nanovg-hs;
      isLibrary = true;
      isExecutable = true;
      libraryHaskellDepends = [ base bytestring containers text vector ];
      librarySystemDepends = [ glew libGL libGLU ];
      libraryPkgconfigDepends = [ glew ];
      libraryToolDepends = [ c2hs ];
      testHaskellDepends = [ base containers hspec inline-c QuickCheck ];
      homepage = "https://github.com/cocreature/nanovg-hs";
      description = "Haskell bindings for nanovg";
      license = stdenv.lib.licenses.isc;
    }
  ) {};

  affection = with haskellPackages; callPackage(
    { mkDerivation, base, bytestring, clock, containers, glib, linear
    , monad-loops, monad-parallel, mtl, OpenGL, stdenv, stm, text
    , uuid, vector
    }:
    mkDerivation {
      pname = "affection";
      version = "0.0.0.10";
      src = ../affection;
      isLibrary = true;
      isExecutable = true;
      libraryHaskellDepends = [
        base bytestring clock containers glib linear monad-loops
        monad-parallel mtl OpenGL sdl2 stm text uuid vector
      ];
      homepage = "https://github.com/nek0/affection#readme";
      description = "A simple Game Engine using SDL";
      license = stdenv.lib.licenses.lgpl3;
    }
  ) {};

  f = { mkDerivation, base, containers, glew
      , linear, OpenGL, random, stdenv, stm
      }:
      mkDerivation {
        pname = "haskelloids";
        version = "0.0.0.1";
        src = ./.;
        isLibrary = false;
        isExecutable = true;
        enableExecutableProfiling = true;
        executableHaskellDepends = [
          affection base containers linear nanovg OpenGL random sdl stm
        ];
        executableSystemDepends = [ glew ];
        executablePkgconfigDepends = [ glew ];
        description = "A little game using Affection";
        license = stdenv.lib.licenses.gpl3;
      };

  haskellPackages = if compiler == "default"
                       then pkgs.haskellPackages
                       else pkgs.haskell.packages.${compiler};

  variant = if doBenchmark then pkgs.haskell.lib.doBenchmark else pkgs.lib.id;

  drv = variant (haskellPackages.callPackage f {});

in

  if pkgs.lib.inNixShell then drv.env else drv
